/*
 * Copyright (c) 1999-2021 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_IMAGE_HPP
#define PW_CORE_IMAGE_HPP

#include <pw/core/globals.hpp>
#include <pw/core/size.hpp>

namespace pw {

class image {
public:

    image() = default;

	using data_t = std::byte;

    enum pixel_layout {
        RGB8,
        RGBA8,
		LUM,
		DEPTH,
		HDR
    };

    image(const size& s, pixel_layout t, const void *ptr = nullptr);

    bool create(const size& s, pixel_layout t, const void *ptr = nullptr);

	void release(bool release_memory = false);

	const data_t *data() const { return _data.data(); }
	data_t *data() { return _data.data(); }

	const float *data_float() const { return reinterpret_cast<const float*>(_data.data()); }
	float *data_float() { return reinterpret_cast<float*>(_data.data());}

    pixel_layout layout() const;
    void set_layout(const pixel_layout &layout);

    uint64_t change_count() const;
    void set_change_count(const uint64_t &change_count);

    static uint32_t bytes_per_pixel(pixel_layout t);

    static uint32_t components(pixel_layout t);

    ::pw::size size() const;

    void generate_noise();
protected:

    ::pw::size _size;
    pixel_layout _layout;
    uint64_t _change_count;

	std::vector<data_t> _data;

};

}

#endif
