/*
 * Copyright (c) 1999-2021 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_MATERIAL_HPP
#define PW_CORE_MATERIAL_HPP


#include <pw/core/vector.hpp>

namespace pw {

class material {
public:
    enum class texture_type {
        diffuse,
        specular,
        ambient,
        emissive,
        height,
        normals,
        shininess,
        opacity,
        displacement,
        lightmap,
        reflection,
        base_color,
        // more to implement ....
    };

    enum class shading_type {
        flat,
        gouraud,
        phong,
        blinn_phong
        // well ...
    };

    material() = default;

    using texture_t = std::tuple<texture_type,std::string,uint32_t>;



protected:

    vector4 _color = vector4 { 1.0, 0.0, 1.0, 1.0 };

    std::vector<texture_t> _textures;

};

}

#endif
