#include "pw/visual/renderer.hpp"

#include "pw/core/geometry.hpp"
#include "pw/core/size.hpp"
#include "pw/core/debug.hpp"

#include "glad/glad.h"

#include <algorithm>


namespace pw {

struct renderer::impl {

    uint32_t _vao { 0 };
    uint32_t _ebo { 0 };
    uint32_t _vbo { 0 };

    GLint _mesh_elements = { 0 };

    impl() = default;

    ~impl()
    {
        release();
    }

    bool ready() const
    {
        return glIsVertexArray != nullptr && GL_TRUE == glIsVertexArray(_vao);
    }

    bool create(const geometry& m)
    {
        // reset if the renderer already in use
        if (ready()) {
            release();
        }

        _mesh_elements = m.indices().size();

        //
        glGenVertexArrays(1,&_vao);
        glGenBuffers(1, &_vbo);
        glGenBuffers(1, &_ebo);

        glBindVertexArray(_vao);

        // TODO binding separate VBOs to the
        // vertexarray should be avoided

        // indices
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, _ebo);
        glBufferData(GL_ELEMENT_ARRAY_BUFFER,
                     m.indices().size() * sizeof (uint32_t),
                     m.indices().data(),
                     GL_STATIC_DRAW);


        // vertices
        glBindBuffer(GL_ARRAY_BUFFER, _vbo);
        glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, nullptr);
        glBufferData(GL_ARRAY_BUFFER,
                     m.vertices().size() * sizeof(vector3),
                     m.vertices().data(),
                     GL_STATIC_DRAW);
        glEnableVertexAttribArray(0);


        // stop binding
        glBindVertexArray(0);


#if 0
        // get errors
        auto error = glGetError();
        if (error != GL_NO_ERROR) {
            debug::e() << __PRETTY_FUNCTION__ << " GL error: " << error;
        }

#endif

        return ready();
    }


    void release()
    {

        glDeleteBuffers(1,&_vbo);
        glDeleteBuffers(1,&_ebo);

        glDeleteVertexArrays(1,&_vao);
    }

    void draw()
    {

#if 1
        glEnable(GL_CULL_FACE);
        glCullFace(GL_BACK);
        glFrontFace(GL_CCW);

        glClearColor(0.0,1.0,0,1);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
#endif



        glBindVertexArray(_vao);
        glDrawElements(GL_TRIANGLES, _mesh_elements, GL_UNSIGNED_INT, nullptr);
        glBindVertexArray(0);

#if 1
        auto error = glGetError();
        if (error != GL_NO_ERROR)
        {
            debug::e() << __PRETTY_FUNCTION__ << " OpenGL error: " << error;
        }
#endif
    }

    //	GLint get_mode(vertex_array::)

};

//
//
//

renderer::renderer()
    : _impl(std::make_unique<renderer::impl>())
{
}

renderer::renderer(const geometry &m)
{
    renderer();
    _impl->create(m);
}

renderer::~renderer()
{
}

bool renderer::ready() const
{
    return _impl->ready();
}

bool renderer::create(const geometry &m)
{
    return _impl->create(m);
}

void renderer::release()
{
    _impl->release();
}

void renderer::draw()
{
    _impl->draw();
}


//class viewport {



//    void set(point p,size s);



//protected:

//    struct _impl;
//    std::unique_ptr<_impl>;

//};


}
